@@ -31,6 +31,7 @@ module Aws.DynamoDb.Core
     , ddbUsWest1
     , ddbUsWest2
     , ddbEuWest1
+    , ddbEuWest2
     , ddbEuCentral1
     , ddbApNe1
     , ddbApSe1
@@ -120,14 +121,14 @@ import qualified Control.Exception            as C
 import           Control.Monad
 import           Control.Monad.Trans
 import           Control.Monad.Trans.Resource (throwM)
-import           Crypto.Hash
+import qualified Crypto.Hash                  as CH
 import           Data.Aeson
 import qualified Data.Aeson                   as A
 import           Data.Aeson.Types             (Pair, parseEither)
 import qualified Data.Aeson.Types             as A
 import qualified Data.Attoparsec.ByteString   as AttoB (endOfInput)
 import qualified Data.Attoparsec.Text         as Atto
-import           Data.Byteable
+import qualified Data.ByteArray               as ByteArray
 import qualified Data.ByteString.Base16       as Base16
 import qualified Data.ByteString.Base64       as Base64
 import qualified Data.ByteString.Char8        as B
@@ -532,6 +533,15 @@ instance ToJSON PrimaryKey where
           Object p2 = toJSON r
       in Object (p1 `HM.union` p2)
 
+instance FromJSON PrimaryKey where
+    parseJSON p = do
+       l <- listPKey p
+       case length l of
+          1 -> return $ head l 
+          _ -> fail "Unable to parse PrimaryKey"     
+      where listPKey p'= map (\(txt,dval)-> hk txt dval)
+                          . HM.toList <$> parseJSON p'
+
 
 -- | A key-value pair
 data Attribute = Attribute {
@@ -784,6 +794,9 @@ ddbUsWest2 = Region "dynamodb.us-west-2.amazonaws.com" "us-west-2"
 ddbEuWest1 :: Region
 ddbEuWest1 = Region "dynamodb.eu-west-1.amazonaws.com" "eu-west-1"
 
+ddbEuWest2 :: Region
+ddbEuWest2 = Region "dynamodb.eu-west-2.amazonaws.com" "eu-west-2"
+
 ddbEuCentral1 :: Region
 ddbEuCentral1 = Region "dynamodb.eu-central-1.amazonaws.com" "eu-central-1"
 
@@ -839,7 +852,7 @@ ddbSignQuery target body di sd
         sigTime = fmtTime "%Y%m%dT%H%M%SZ" $ signatureTime sd
 
         bodyLBS = A.encode body
-        bodyHash = Base16.encode $ toBytes (hashlazy bodyLBS :: Digest SHA256)
+        bodyHash = Base16.encode $ ByteArray.convert (CH.hashlazy bodyLBS :: CH.Digest CH.SHA256)
 
         -- for some reason AWS doesn't want the x-amz-security-token in the canonical request
         amzHeaders = [ ("x-amz-date", sigTime)
